@@ -1,4 +1,4 @@
-/* $Id: rmagick.h,v 1.105 2006/02/19 17:12:20 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.116 2006/06/28 23:05:31 rmagick Exp $ */
 /*=============================================================================
 |               Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmagick.h
@@ -115,8 +115,7 @@ typedef enum
 {
     False = 0,
     True = 1
-}
-boolean;
+} rm_boolean;
 
 typedef enum {
     AnyWeight,
@@ -211,6 +210,22 @@ typedef unsigned int MagickBooleanType;
 #define UndefinedGravity 0
 #endif
 
+
+#if !defined(HAVE_SETIMAGEINFOFILE)
+#define SetImageInfoFile(info, fptr) (info)->file = (fptr)
+#endif
+
+#if !defined(HAVE_SETIMAGEEXTENT)
+#define SetImageExtent(img, c, r) \
+    (img)->columns = (c); \
+    (img)->rows = (r)
+#endif
+
+
+#if !defined(HAVE_SETIMAGEBACKGROUNDCOLOR)
+#define SetImageBackgroundColor(img) SetImage((img), OpaqueOpacity)
+#endif
+
 // This implements the "omitted storage class model" for external variables.
 // (Ref: Harbison & Steele.) The rmmain.c file defines MAIN, which causes
 // the single defining declarations to be generated. No other source files
@@ -274,6 +289,9 @@ EXTERN VALUE Class_FilterTypes;
 EXTERN VALUE Class_GravityType;
 EXTERN VALUE Class_ImageType;
 EXTERN VALUE Class_InterlaceType;
+#if defined(HAVE_COMPAREIMAGELAYERS)
+EXTERN VALUE Class_MagickLayerMethod;
+#endif
 EXTERN VALUE Class_NoiseType;
 #if defined(HAVE_IMAGE_ORIENTATION)
 EXTERN VALUE Class_OrientationType;
@@ -324,14 +342,8 @@ EXTERN ID ID_y;                 // "y"
 
 /*
    Handle warnings & errors
-
-   For ExceptionInfo structures in auto storage. Destroys the
-   ExceptionInfo structure (releasing any IM storage) before
-   calling rb_raise.
 */
-#define HANDLE_ERROR rm_handle_error(&exception);
-// For ExceptionInfo structures in images.
-#define HANDLE_ERROR_IMG(img) rm_handle_error(&((img)->exception));
+#define CHECK_EXCEPTION() rm_check_exception(&exception, NULL, RetainOnError);
 
 /*
     Map the QuantumDepth to a StorageType.
@@ -593,6 +605,7 @@ extern VALUE ImageList_map(VALUE, VALUE, VALUE);
 extern VALUE ImageList_montage(VALUE);
 extern VALUE ImageList_morph(VALUE, VALUE);
 extern VALUE ImageList_mosaic(VALUE);
+extern VALUE ImageList_optimize_layers(VALUE, VALUE);
 extern VALUE ImageList_quantize(int, VALUE*, VALUE);
 extern VALUE ImageList_to_blob(VALUE);
 extern VALUE ImageList_write(VALUE, VALUE);
@@ -710,7 +723,7 @@ ATTR_READER(Image, normalized_maximum_error)
 ATTR_READER(Image, number_colors)
 ATTR_ACCESSOR(Image, offset)
 ATTR_WRITER(Image, opacity)
-ATTR_READER(Image, orientation)
+ATTR_ACCESSOR(Image, orientation)
 ATTR_ACCESSOR(Image, page)
 ATTR_READER(Image, quality)
 ATTR_READER(Image, quantum_depth)
@@ -734,12 +747,16 @@ extern VALUE Image_new(int, VALUE *, VALUE);
 extern VALUE Image_initialize(VALUE, VALUE, VALUE, VALUE, VALUE);
 #endif
 
+extern VALUE Image_adaptive_sharpen(int, VALUE *, VALUE);
+extern VALUE Image_adaptive_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_adaptive_threshold(int, VALUE *, VALUE);
 extern VALUE Image_add_noise(VALUE, VALUE);
 extern VALUE Image_add_noise_channel(int, VALUE *, VALUE);
 extern VALUE Image_affine_transform(VALUE, VALUE);
 extern VALUE Image_aref(VALUE, VALUE);
 extern VALUE Image_aset(VALUE, VALUE, VALUE);
+extern VALUE Image_auto_orient(VALUE);
+extern VALUE Image_auto_orient_bang(VALUE);
 extern VALUE Image_properties(VALUE);
 extern VALUE Image_bilevel_channel(int, VALUE *, VALUE);
 extern VALUE Image_black_threshold(int, VALUE *, VALUE);
@@ -765,10 +782,13 @@ extern VALUE Image_colorize(int, VALUE *, VALUE);
 extern VALUE Image_colormap(int, VALUE *, VALUE);
 extern VALUE Image_composite(int, VALUE *, VALUE);
 extern VALUE Image_composite_bang(int, VALUE *, VALUE);
+extern VALUE Image_composite_channel(int, VALUE *, VALUE);
+extern VALUE Image_composite_channel_bang(int, VALUE *, VALUE);
 extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
 extern VALUE Image_compress_colormap_bang(VALUE);
 extern VALUE Image_constitute(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_contrast(int, VALUE *, VALUE);
+extern VALUE Image_contrast_stretch_channel(int, VALUE *, VALUE);
 extern VALUE Image_convolve(VALUE, VALUE, VALUE);
 extern VALUE Image_convolve_channel(int, VALUE *, VALUE);
 extern VALUE Image_copy(VALUE);
@@ -807,7 +827,7 @@ extern VALUE Image_implode(int, VALUE *, VALUE);
 extern VALUE Image_import_pixels(int, VALUE *, VALUE);
 extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
-extern VALUE Image_level(int, VALUE *, VALUE);
+extern VALUE Image_level2(int, VALUE *, VALUE);
 extern VALUE Image_level_channel(int, VALUE *, VALUE);
 extern VALUE Image__load(VALUE, VALUE);
 extern VALUE Image_magnify(VALUE);
@@ -839,6 +859,7 @@ extern VALUE Image_quantize(int, VALUE *, VALUE);
 extern VALUE Image_quantization_error(VALUE);
 extern VALUE Image_quantum_operator(int, VALUE *, VALUE);
 extern VALUE Image_radial_blur(VALUE, VALUE);
+extern VALUE Image_radial_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_raise(int, VALUE *, VALUE);
 extern VALUE Image_random_channel_threshold(VALUE, VALUE, VALUE);
 extern VALUE Image_random_threshold_channel(int, VALUE *, VALUE);
@@ -884,6 +905,10 @@ extern VALUE Image_tint(int, VALUE *, VALUE);
 extern VALUE Image_to_blob(VALUE);
 extern VALUE Image_to_color(VALUE, VALUE);
 extern VALUE Image_transparent(int, VALUE *, VALUE);
+extern VALUE Image_transpose(VALUE);
+extern VALUE Image_transpose_bang(VALUE);
+extern VALUE Image_transverse(VALUE);
+extern VALUE Image_transverse_bang(VALUE);
 extern VALUE Image_trim(VALUE);
 extern VALUE Image_trim_bang(VALUE);
 extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
@@ -951,9 +976,11 @@ ATTR_ACCESSOR(Pixel, black)
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
 extern VALUE  Pixel_dup(VALUE);
+extern VALUE  Pixel_eql_q(VALUE, VALUE);
 extern VALUE  Pixel_fcmp(int, VALUE *, VALUE);
 extern VALUE  Pixel_from_color(VALUE, VALUE);
 extern VALUE  Pixel_from_HSL(VALUE, VALUE);
+extern VALUE  Pixel_hash(VALUE);
 extern VALUE  Pixel_initialize(int, VALUE *, VALUE);
 extern VALUE  Pixel_init_copy(VALUE, VALUE);
 extern VALUE  Pixel_intensity(VALUE);
@@ -1022,18 +1049,28 @@ extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
 extern int    rm_strcasecmp(const char *, const char *);
 extern void   rm_check_ary_len(VALUE, int);
 extern void   rm_check_frozen(VALUE);
+extern int    rm_check_num2dbl(VALUE);
 extern double rm_fuzz_to_dbl(VALUE);
 extern double rm_percentage(VALUE);
+extern double rm_str_to_pct(VALUE);
 extern VALUE  rm_define_enum_type(char *);
 extern void   rm_write_temp_image(Image *, char *);
 extern void   rm_delete_temp_image(char *);
 extern void   rm_not_implemented(void);
-extern void   rm_handle_error(ExceptionInfo *);
-extern void   rm_handle_all_errors(Image *);
 extern void   rm_attr_write(VALUE, VALUE);
 extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
 extern void   rm_split(Image *);
 
+typedef enum
+{
+    RetainOnError = 0,
+    DestroyOnError = 1
+} ErrorRetention;
+
+extern void   rm_check_image_exception(Image *, ErrorRetention);
+extern void   rm_check_exception(ExceptionInfo *, Image *, ErrorRetention);
+extern void   rm_ensure_result(Image *);
+extern Image *rm_clone_image(Image *);
 #if defined(HAVE_SETIMAGEPROGRESSMONITOR)
 extern MagickBooleanType rm_progress_monitor(const char *, const MagickOffsetType, const MagickSizeType, void *);
 #endif
