@@ -1,4 +1,4 @@
-/* $Id: rmilist.c,v 1.31 2006/02/03 23:26:05 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.41 2006/06/20 23:33:07 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmilist.c
@@ -48,7 +48,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     Data_Get_Struct(info_obj, Info, info);
 
     (void) AnimateImages(info, images);
-    rm_handle_all_errors(images);
+    rm_check_image_exception(images, RetainOnError);
     rm_split(images);
 
     return self;
@@ -62,7 +62,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
 VALUE
 ImageList_append(VALUE self, VALUE stack_arg)
 {
-    Image *images, *result;
+    Image *images, *new_image;
     unsigned int stack;
     ExceptionInfo exception;
 
@@ -74,11 +74,14 @@ ImageList_append(VALUE self, VALUE stack_arg)
     stack = RTEST(stack_arg);
 
     GetExceptionInfo(&exception);
-    result = AppendImages(images, stack, &exception);
+    new_image = AppendImages(images, stack, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    DestroyExceptionInfo(&exception);
 
-    return rm_image_new(result);
+    rm_ensure_result(new_image);
+
+    return rm_image_new(new_image);
 }
 
 /*
@@ -89,18 +92,21 @@ ImageList_append(VALUE self, VALUE stack_arg)
 VALUE
 ImageList_average(VALUE self)
 {
-    Image *images, *result;
+    Image *images, *new_image;
     ExceptionInfo exception;
 
     // Convert the images array to an images sequence.
     images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
-    result = AverageImages(images, &exception);
-    rm_handle_all_errors(images);
+    new_image = AverageImages(images, &exception);
     rm_split(images);
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    DestroyExceptionInfo(&exception);
 
-    return rm_image_new(result);
+    rm_ensure_result(new_image);
+
+    return rm_image_new(new_image);
 }
 
 /*
@@ -114,18 +120,21 @@ ImageList_average(VALUE self)
 VALUE
 ImageList_coalesce(VALUE self)
 {
-    Image *images, *results;
+    Image *images, *new_images;
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
     images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
-    results = CoalesceImages(images, &exception);
+    new_images = CoalesceImages(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
 
-    return rm_imagelist_from_images(results);
+    rm_ensure_result(new_images);
+
+    return rm_imagelist_from_images(new_images);
 }
 
 
@@ -146,7 +155,10 @@ ImageList_deconstruct(VALUE self)
     GetExceptionInfo(&exception);
     new_images = DeconstructImages(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_images);
 
     return rm_imagelist_from_images(new_images);
 }
@@ -161,7 +173,6 @@ ImageList_display(VALUE self)
     Image *images;
     Info *info;
     volatile VALUE info_obj;
-    unsigned int ok;
 
     // Convert the images array to an images sequence.
     images = rm_images_from_imagelist(self);
@@ -170,12 +181,9 @@ ImageList_display(VALUE self)
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
 
-    ok = DisplayImages(info, images);
-    if (!ok)
-    {
-        rm_handle_all_errors(images);
-    }
+    (void) DisplayImages(info, images);
     rm_split(images);
+    rm_check_image_exception(images, RetainOnError);
 
     return self;
 }
@@ -196,12 +204,16 @@ ImageList_flatten_images(VALUE self)
     GetExceptionInfo(&exception);
     new_image = FlattenImages(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_image);
 
     return rm_image_new(new_image);
 }
 
 
+
 /*
     Method:     ImageList#map
     Purpose:    Call MapImages
@@ -210,7 +222,7 @@ ImageList_flatten_images(VALUE self)
 VALUE
 ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
 {
-    Image *images, *clone_images = NULL;
+    Image *images, *new_images = NULL;
     Image *map;
     unsigned int dither;
     volatile VALUE image, scene, new_imagelist;
@@ -225,19 +237,23 @@ ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
     }
 
     // Convert image array to image sequence, clone image sequence.
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    clone_images = CloneImageList(images, &exception);
+
+    images = rm_images_from_imagelist(self);
+    new_images = CloneImageList(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_images);
 
     // Call ImageMagick
     dither = !(dither_arg == Qfalse || dither_arg == Qnil);
-    (void) MapImages(clone_images, map, dither);
-    HANDLE_ERROR_IMG(clone_images)
+    (void) MapImages(new_images, map, dither);
+    rm_check_image_exception(new_images, DestroyOnError);
 
     // Set @scene in new ImageList object to same value as in self.
-    new_imagelist = rm_imagelist_from_images(clone_images);
+    new_imagelist = rm_imagelist_from_images(new_images);
     scene = rb_iv_get(self, "@scene");
     (void) rm_imagelist_scene_eq(new_imagelist, scene);
 
@@ -255,7 +271,7 @@ ImageList_montage(VALUE self)
 {
     volatile VALUE montage_obj;
     Montage *montage;
-    Image *montage_seq, *image_list;
+    Image *new_images, *images;
     ExceptionInfo exception;
 
     // Create a new instance of the Magick::Montage class
@@ -269,13 +285,13 @@ ImageList_montage(VALUE self)
 
     Data_Get_Struct(montage_obj, Montage, montage);
 
-    image_list = rm_images_from_imagelist(self);
+    images = rm_images_from_imagelist(self);
 
     // If app specified a non-default composition operator, use it for all images.
     if (montage->compose != UndefinedCompositeOp)
     {
         Image *i;
-        for (i = image_list; i; i = GetNextImageInList(i))
+        for (i = images; i; i = GetNextImageInList(i))
         {
             i->compose = montage->compose;
         }
@@ -284,11 +300,14 @@ ImageList_montage(VALUE self)
     GetExceptionInfo(&exception);
 
     // MontageImage can return more than one image.
-    montage_seq = MontageImages(image_list, montage->info, &exception);
-    rm_split(image_list);
-    HANDLE_ERROR
+    new_images = MontageImages(images, montage->info, &exception);
+    rm_split(images);
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
 
-    return rm_imagelist_from_images(montage_seq);
+    rm_ensure_result(new_images);
+
+    return rm_imagelist_from_images(new_images);
 }
 
 /*
@@ -322,7 +341,10 @@ ImageList_morph(VALUE self, VALUE nimages)
     GetExceptionInfo(&exception);
     new_images = MorphImages(images, (unsigned long)number_images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_images);
 
     return rm_imagelist_from_images(new_images);
 }
@@ -342,11 +364,68 @@ ImageList_mosaic(VALUE self)
     GetExceptionInfo(&exception);
     new_image = MosaicImages(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_image);
 
     return rm_image_new(new_image);
 }
 
+
+/*
+    Method:     ImageList#optimize_layers
+    Purpose:    Equivalent to -layers option in 6.2.6
+    Returns:    a new imagelist
+*/
+VALUE
+ImageList_optimize_layers(VALUE self, VALUE method)
+{
+#if defined(HAVE_COMPAREIMAGELAYERS)
+    Image *images, *new_images;
+    MagickLayerMethod mthd;
+    ExceptionInfo exception;
+
+    images = rm_images_from_imagelist(self);
+    GetExceptionInfo(&exception);
+    VALUE_TO_ENUM(method, mthd, MagickLayerMethod);
+
+    switch (mthd)
+    {
+#if defined(HAVE_COALESCELAYER)
+      case CoalesceLayer:
+        new_images = CoalesceImages(images, &exception);
+        break;
+      case DisposeLayer:
+        new_images = DisposeImages(images, &exception);
+        break;
+#endif
+      case OptimizeLayer:
+        new_images = OptimizeImageLayers(images, &exception);
+        break;
+      case OptimizePlusLayer:
+        new_images = OptimizePlusImageLayers(images, &exception);
+        break;
+      default:
+        new_images = CompareImageLayers(images, mthd, &exception);
+        break;
+    }
+
+    rm_split(images);
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_images);
+
+    return rm_imagelist_from_images(new_images);
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 /*
     External:   rm_imagelist_new
     Purpose:    create a new ImageList object with no images
@@ -371,6 +450,11 @@ rm_imagelist_from_images(Image *images)
      volatile VALUE new_imagelist;
      Image *image;
 
+     if (!images)
+     {
+         rb_bug("rm_imagelist_from_images called with NULL argument");
+     }
+
      new_imagelist = rm_imagelist_new();
 
      while (images)
@@ -498,9 +582,14 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     images = rm_images_from_imagelist(self);
     new_images = CloneImageList(images, &exception);
     rm_split(images);
-    HANDLE_ERROR
+    rm_check_exception(&exception, new_images, DestroyOnError);
+    DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_images);
+
 
     QuantizeImages(&quantize_info, new_images);
+    rm_check_exception(&exception, new_images, DestroyOnError);
 
     // Create new ImageList object, convert mapped image sequence to images,
     // append to images array.
@@ -529,7 +618,7 @@ ImageList_to_blob(VALUE self)
     Image *images;
     Info *info;
     volatile VALUE info_obj;
-	volatile VALUE blob_str;
+    volatile VALUE blob_str;
     void *blob = NULL;
     size_t length = 0;
     ExceptionInfo exception;
@@ -542,7 +631,8 @@ ImageList_to_blob(VALUE self)
 
     GetExceptionInfo(&exception);
     (void) SetImageInfo(info, True, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+
     if (*info->magick != '\0')
     {
         Image *img;
@@ -556,14 +646,19 @@ ImageList_to_blob(VALUE self)
     // can happen is that there's only one image or the format
     // doesn't support multi-image files.
     info->adjoin = True;
-    GetExceptionInfo(&exception);
 #if defined(HAVE_IMAGESTOBLOB)
     blob = ImagesToBlob(info, images, &length, &exception);
 #else
     blob = ImageToBlob(info, images, &length, &exception);
 #endif
+    if (exception.severity != UndefinedException)
+    {
+        magick_free((void*)blob);
+    }
     rm_split(images);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 
     if (length == 0 || !blob)
     {
@@ -571,7 +666,7 @@ ImageList_to_blob(VALUE self)
     }
 
     blob_str = rb_str_new(blob, length);
- 	magick_free((void*)blob);
+    magick_free((void*)blob);
 
     return blob_str;
 }
@@ -621,7 +716,7 @@ ImageList_write(VALUE self, VALUE file)
 
         // Ensure file is open - raise error if not
         GetOpenFile(file, fptr);
-        info->file = GetReadFile(fptr);
+        SetImageInfoFile(info, GetReadFile(fptr));
     }
     else
     {
@@ -633,7 +728,7 @@ ImageList_write(VALUE self, VALUE file)
         filenameL = min(filenameL, MaxTextExtent-1);
         memcpy(info->filename, filename, (size_t)filenameL);
         info->filename[filenameL] = '\0';
-        info->file = NULL;
+        SetImageInfoFile(info, NULL);
     }
 
     // Copy the filename into each images. Set a scene number to be used if
@@ -649,7 +744,8 @@ ImageList_write(VALUE self, VALUE file)
 
     // Find out if the format supports multi-images files.
     m = GetMagickInfo(info->magick, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
 
     // Tell WriteImage if we want a multi-images file.
     if (rm_imagelist_length(self) > 1 && m->adjoin)
@@ -661,7 +757,7 @@ ImageList_write(VALUE self, VALUE file)
     {
         (void) WriteImage(info, img);
         // images will be split before raising an exception
-        rm_handle_all_errors(images);
+        rm_check_image_exception(images, RetainOnError);
         if (info->adjoin)
         {
             break;
