@@ -1,4 +1,4 @@
-/* $Id: rmmain.c,v 1.111 2006/02/19 17:12:20 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.124 2006/06/26 23:32:37 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmmain.c
@@ -43,7 +43,9 @@ Magick_colors(VALUE class)
     GetExceptionInfo(&exception);
 
     color_ary = GetColorInfoArray(&exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 
     if (rb_block_given_p())
     {
@@ -86,7 +88,9 @@ Magick_colors(VALUE class)
     GetExceptionInfo(&exception);
 
     color_info_list = GetColorInfoList("*", &number_colors, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
 
@@ -174,7 +178,9 @@ Magick_fonts(VALUE class)
 
     GetExceptionInfo(&exception);
     type_info = GetTypeInfoList("*", &number_types, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
     if (rb_block_given_p())
@@ -207,7 +213,8 @@ Magick_fonts(VALUE class)
     GetExceptionInfo(&exception);
 
     type_list = GetTypeInfo("*", &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
 
     // If block, iterate over fonts
     if (rb_block_given_p())
@@ -280,6 +287,9 @@ Magick_init_formats(VALUE class)
 
     GetExceptionInfo(&exception);
     magick_info = (MagickInfo *)GetMagickInfoArray(&exception);
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
     for(m = magick_info; m != NULL; m = m->next)
     {
         rb_hash_aset(formats, rb_str_new2(m->name), MagickInfo_to_format(m));
@@ -307,7 +317,9 @@ Magick_init_formats(VALUE class)
     // IM 6.1.3 added an exception argument to GetMagickInfoList
     GetExceptionInfo(&exception);
     magick_info = GetMagickInfoList("*", &number_formats, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+
 #endif
 
     for(x = 0; x < number_formats; x++)
@@ -330,7 +342,8 @@ Magick_init_formats(VALUE class)
 
     GetExceptionInfo(&exception);
     m = (MagickInfo *)GetMagickInfo("*", &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
 
     for ( ; m != NULL; m = m->next)
     {
@@ -342,6 +355,98 @@ Magick_init_formats(VALUE class)
 #endif
 }
 
+
+/*
+  Method:   Magick.limit_resource(resource[, limit])
+  Purpose:  Get/set resource limits. If a limit is specified the old limit
+            is set to the new value. Either way the current/new limit is returned.
+  Notes:    Don't support "AreaLimit" because GraphicsMagick doesn't support it.
+*/
+static VALUE
+Magick_limit_resource(int argc, VALUE *argv, VALUE class)
+{
+#if defined(HAVE_GETMAGICKRESOURCELIMIT)
+    volatile VALUE resource, limit;
+    ResourceType res = UndefinedResource;
+    char *str;
+    ID id;
+    unsigned long cur_limit;
+
+    rb_scan_args(argc, argv, "11", &resource, &limit);
+
+    switch (TYPE(resource))
+    {
+        case T_NIL:
+            return class;
+
+        case T_SYMBOL:
+            id = SYM2ID(resource);
+            if (id == rb_intern("memory"))
+            {
+                res = MemoryResource;
+            }
+            else if (id == rb_intern("map"))
+            {
+                res = MapResource;
+            }
+            else if (id == rb_intern("disk"))
+            {
+                res = DiskResource;
+            }
+            else if (id == rb_intern("file"))
+            {
+                res = FileResource;
+            }
+            else
+            {
+                rb_raise(rb_eArgError, "unknown resource: `:%s'", rb_id2name(id));
+            }
+            break;
+
+        default:
+            str = STRING_PTR(resource);
+            if (*str == '\0')
+            {
+                return class;
+            }
+            else if (rm_strcasecmp("memory", str) == 0)
+            {
+                res = MemoryResource;
+            }
+            else if (rm_strcasecmp("map", str) == 0)
+            {
+                res = MapResource;
+            }
+            else if (rm_strcasecmp("disk", str) == 0)
+            {
+                res = DiskResource;
+            }
+            else if (rm_strcasecmp("file", str) == 0)
+            {
+                res = FileResource;
+            }
+            else
+            {
+                rb_raise(rb_eArgError, "unknown resource: `%s'", str);
+            }
+            break;
+    }
+
+    cur_limit = GetMagickResourceLimit(res);
+
+    if (argc > 1)
+    {
+        SetMagickResourceLimit(res, NUM2ULONG(limit));
+    }
+
+    return ULONG2NUM(cur_limit);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 /*
     This is the exit known to ImageMagick. Retrieve the monitor
     proc and call it, passing the 3 exit arguments.
@@ -530,6 +635,7 @@ Init_RMagick(void)
     rb_define_module_function(Module_Magick, "colors", Magick_colors, 0);
     rb_define_module_function(Module_Magick, "fonts", Magick_fonts, 0);
     rb_define_module_function(Module_Magick, "init_formats", Magick_init_formats, 0);
+    rb_define_module_function(Module_Magick, "limit_resource", Magick_limit_resource, -1);
     rb_define_module_function(Module_Magick, "set_monitor", Magick_set_monitor, 1);
     rb_define_module_function(Module_Magick, "set_cache_threshold", Magick_set_cache_threshold, 1);
     rb_define_module_function(Module_Magick, "set_log_event_mask", Magick_set_log_event_mask, -1);
@@ -604,7 +710,7 @@ Init_RMagick(void)
     DCL_ATTR_READER(Image, number_colors)
     DCL_ATTR_ACCESSOR(Image, offset)
     DCL_ATTR_WRITER(Image, opacity)
-    DCL_ATTR_READER(Image, orientation)
+    DCL_ATTR_ACCESSOR(Image, orientation)
     DCL_ATTR_ACCESSOR(Image, page)
 #if defined(HAVE_IMAGE_QUALITY)
     DCL_ATTR_READER(Image, quality)
@@ -623,12 +729,16 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Image, x_resolution)
     DCL_ATTR_ACCESSOR(Image, y_resolution)
 
+    rb_define_method(Class_Image, "adaptive_sharpen", Image_adaptive_sharpen, -1);
+    rb_define_method(Class_Image, "adaptive_sharpen_channel", Image_adaptive_sharpen_channel, -1);
     rb_define_method(Class_Image, "adaptive_threshold", Image_adaptive_threshold, -1);
     rb_define_method(Class_Image, "add_noise", Image_add_noise, 1);
     rb_define_method(Class_Image, "add_noise_channel", Image_add_noise_channel, -1);
     rb_define_method(Class_Image, "affine_transform", Image_affine_transform, 1);
     rb_define_method(Class_Image, "[]", Image_aref, 1);
     rb_define_method(Class_Image, "[]=", Image_aset, 2);
+    rb_define_method(Class_Image, "auto_orient", Image_auto_orient, 0);
+    rb_define_method(Class_Image, "auto_orient!", Image_auto_orient_bang, 0);
     rb_define_method(Class_Image, "properties", Image_properties, 0);
     rb_define_method(Class_Image, "bilevel_channel", Image_bilevel_channel, -1);
     rb_define_method(Class_Image, "black_threshold", Image_black_threshold, -1);
@@ -657,8 +767,11 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "composite", Image_composite, -1);
     rb_define_method(Class_Image, "composite!", Image_composite_bang, -1);
     rb_define_method(Class_Image, "composite_affine", Image_composite_affine, 2);
+    rb_define_method(Class_Image, "composite_channel", Image_composite_channel, -1);
+    rb_define_method(Class_Image, "composite_channel!", Image_composite_channel_bang, -1);
     rb_define_method(Class_Image, "compress_colormap!", Image_compress_colormap_bang, 0);
     rb_define_method(Class_Image, "contrast", Image_contrast, -1);
+    rb_define_method(Class_Image, "contrast_stretch_channel", Image_contrast_stretch_channel, -1);
     rb_define_method(Class_Image, "convolve", Image_convolve, 2);
     rb_define_method(Class_Image, "convolve_channel", Image_convolve_channel, -1);
     rb_define_method(Class_Image, "copy", Image_copy, 0);
@@ -697,7 +810,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "import_pixels", Image_import_pixels, -1);
     rb_define_method(Class_Image, "initialize_copy", Image_init_copy, 1);
     rb_define_method(Class_Image, "inspect", Image_inspect, 0);
-    rb_define_method(Class_Image, "level", Image_level, -1);
+    rb_define_method(Class_Image, "level2", Image_level2, -1);
     rb_define_method(Class_Image, "level_channel", Image_level_channel, -1);
     rb_define_method(Class_Image, "magnify", Image_magnify, 0);
     rb_define_method(Class_Image, "magnify!", Image_magnify_bang, 0);
@@ -726,6 +839,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "quantize", Image_quantize, -1);
     rb_define_method(Class_Image, "quantum_operator", Image_quantum_operator, -1);
     rb_define_method(Class_Image, "radial_blur", Image_radial_blur, 1);
+    rb_define_method(Class_Image, "radial_blur_channel", Image_radial_blur_channel, -1);
     rb_define_method(Class_Image, "raise", Image_raise, -1);
     rb_define_method(Class_Image, "random_channel_threshold", Image_random_channel_threshold, 2);
     rb_define_method(Class_Image, "random_threshold_channel", Image_random_threshold_channel, -1);
@@ -769,6 +883,10 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "to_color", Image_to_color, 1);
     rb_define_method(Class_Image, "to_blob", Image_to_blob, 0);
     rb_define_method(Class_Image, "transparent", Image_transparent, -1);
+    rb_define_method(Class_Image, "transpose", Image_transpose, 0);
+    rb_define_method(Class_Image, "transpose!", Image_transpose_bang, 0);
+    rb_define_method(Class_Image, "transverse", Image_transverse, 0);
+    rb_define_method(Class_Image, "transverse!", Image_transverse_bang, 0);
     rb_define_method(Class_Image, "trim", Image_trim, 0);
     rb_define_method(Class_Image, "trim!", Image_trim_bang, 0);
     rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, -1);
@@ -801,6 +919,7 @@ Init_RMagick(void)
     rb_define_method(Class_ImageList, "montage", ImageList_montage, 0);
     rb_define_method(Class_ImageList, "morph", ImageList_morph, 1);
     rb_define_method(Class_ImageList, "mosaic", ImageList_mosaic, 0);
+    rb_define_method(Class_ImageList, "optimize_layers", ImageList_optimize_layers, 1);
     rb_define_method(Class_ImageList, "quantize", ImageList_quantize, -1);
     rb_define_method(Class_ImageList, "to_blob", ImageList_to_blob, 0);
     rb_define_method(Class_ImageList, "write", ImageList_write, 1);
@@ -818,7 +937,7 @@ Init_RMagick(void)
 
     DCL_ATTR_WRITER(Draw, affine)
     DCL_ATTR_WRITER(Draw, align)
-	DCL_ATTR_WRITER(Draw, border_color)
+    DCL_ATTR_WRITER(Draw, border_color)
     DCL_ATTR_WRITER(Draw, decorate)
     DCL_ATTR_WRITER(Draw, density)
     DCL_ATTR_WRITER(Draw, encoding)
@@ -887,11 +1006,13 @@ Init_RMagick(void)
     // Define the instance methods
     rb_define_method(Class_Pixel, "<=>", Pixel_spaceship, 1);
     rb_define_method(Class_Pixel, "===", Pixel_case_eq, 1);
+    rb_define_method(Class_Pixel, "eql?", Pixel_eql_q, 1);
     rb_define_method(Class_Pixel, "initialize", Pixel_initialize, -1);
     rb_define_method(Class_Pixel, "initialize_copy", Pixel_init_copy, 1);
     rb_define_method(Class_Pixel, "clone", Pixel_clone, 0);
     rb_define_method(Class_Pixel, "dup", Pixel_dup, 0);
     rb_define_method(Class_Pixel, "fcmp", Pixel_fcmp, -1);
+    rb_define_method(Class_Pixel, "hash", Pixel_hash, 0);
     rb_define_method(Class_Pixel, "intensity", Pixel_intensity, 0);
     rb_define_method(Class_Pixel, "to_color", Pixel_to_color, -1);
     rb_define_method(Class_Pixel, "to_HSL", Pixel_to_HSL, 0);
@@ -1340,6 +1461,22 @@ Init_RMagick(void)
         ENUMERATOR(PartitionInterlace)
     END_ENUM
 
+#if defined(HAVE_COMPAREIMAGELAYERS)
+    DEF_ENUM(MagickLayerMethod)
+        ENUMERATOR(UndefinedLayer)
+        ENUMERATOR(CompareAnyLayer)
+        ENUMERATOR(CompareClearLayer)
+        ENUMERATOR(CompareOverlayLayer)
+        ENUMERATOR(OptimizeLayer)
+        ENUMERATOR(OptimizePlusLayer)
+#if defined(HAVE_COALESCELAYER)
+        // Introduced in IM 6.2.7-0
+        ENUMERATOR(CoalesceLayer)
+        ENUMERATOR(DisposeLayer)
+#endif
+    END_ENUM
+#endif
+
 #if defined(HAVE_COMPAREIMAGECHANNELS)
     DEF_ENUM(MetricType)
         ENUMERATOR(UndefinedMetric)
@@ -1662,7 +1799,7 @@ static void version_constants(void)
     rb_define_const(Module_Magick, "Version", str);
 
     sprintf(long_version,
-        "This is %s ($Date: 2006/02/19 17:12:20 $) Copyright (C) 2006 by Timothy P. Hunter\n"
+        "This is %s ($Date: 2006/06/26 23:32:37 $) Copyright (C) 2006 by Timothy P. Hunter\n"
         "Built with %s\n"
         "Built for %s\n"
         "Web page: http://rmagick.rubyforge.org\n"
