@@ -1,4 +1,4 @@
-# $Id: RMagick.rb,v 1.37 2005/12/31 15:33:43 rmagick Exp $
+# $Id: RMagick.rb,v 1.43 2006/06/20 23:18:22 rmagick Exp $
 #==============================================================================
 #                  Copyright (C) 2006 by Timothy P. Hunter
 #   Name:       RMagick.rb
@@ -643,22 +643,96 @@ class Image
         copy.crop_resized!(ncols, nrows, gravity)
     end
 
-	def crop_resized!(ncols, nrows, gravity=CenterGravity)
-		if ncols != columns || nrows != rows
-		    scale = [ncols/columns.to_f, nrows/rows.to_f].max
-			resize!(scale*columns+0.5, scale*rows+0.5)
-		end
-	    crop!(gravity, ncols, nrows, true) if ncols != columns || nrows != rows
-	    self
-	end
+    def crop_resized!(ncols, nrows, gravity=CenterGravity)
+        if ncols != columns || nrows != rows
+            scale = [ncols/columns.to_f, nrows/rows.to_f].max
+            resize!(scale*columns+0.5, scale*rows+0.5)
+        end
+        crop!(gravity, ncols, nrows, true) if ncols != columns || nrows != rows
+        self
+    end
 
     # Used by ImageList methods - see ImageList#cur_image
     def cur_image
         self
     end
 
-    # These four methods are equivalent to the Draw#matte
-    # method with the "Point", "Replace", "Floodfill", "FilltoBorder", and
+    # Retrieve EXIF data by entry or all. If one or more entry names specified,
+    # return the values associated with the entries. If no entries specified,
+    # return all entries and values. The return value is an array of [name,value]
+    # arrays.
+    def get_exif_by_entry(*entry)
+        ary = Array.new
+        if entry.length == 0
+            exif_data = self['EXIF:*']
+            if exif_data
+                exif_data.split("\n").each { |exif| ary.push(exif.split('=')) }
+            end
+        else
+            entry.each do |name|
+                rval = self["EXIF:#{name}"]
+                ary.push([name, rval])
+            end
+        end
+        return ary
+    end
+
+    # Retrieve EXIF data by tag number or all tag/value pairs. The return value is a hash.
+    def get_exif_by_number(*tag)
+        hash = Hash.new
+        if tag.length == 0
+            exif_data = self['EXIF:!']
+            if exif_data
+                exif_data.split("\n").each do |exif|
+                    tag, value = exif.split('=')
+                    tag = tag[1,4].hex
+                    hash[tag] = value
+                end
+            end
+        else
+            tag.each do |num|
+                rval = self["EXIF:#{'#%04X' % num}"]
+                hash[num] = rval == 'unknown' ? nil : rval
+            end
+        end
+        return hash
+    end
+
+    # Patches problematic change to the order of arguments in 1.11.0.
+    # Before this release, the order was
+    #       black_point, gamma, white_point
+    # RMagick 1.11.0 changed this to
+    #       black_point, white_point, gamma
+    # This fix tries to determine if the arguments are in the old order and
+    # if so, swaps the gamma and white_point arguments.  Then it calls
+    # level2, which simply accepts the arguments as given.
+
+    # Inspect the gamma and white point values and swap them if they
+    # look like they're in the old order.
+
+    # (Thanks to Al Evans for the suggestion.)
+    def level(black_point=0.0, white_point=nil, gamma=nil)
+        black_point = Float(black_point)
+
+        white_point ||= Magick::MaxRGB - black_point
+        white_point = Float(white_point)
+
+        gamma_arg = gamma
+        gamma ||= 1.0
+        gamma = Float(gamma)
+
+        if gamma.abs > 10.0 || white_point.abs <= 10.0 || white_point.abs < gamma.abs
+            gamma, white_point = white_point, gamma
+            unless gamma_arg
+                white_point = Magick::MaxRGB - black_point
+            end
+        end
+
+        return level2(black_point, white_point, gamma)
+    end
+
+    # These four methods are equivalent to the Draw#matte method
+    # with the "Point", "Replace", "Floodfill", "FilltoBorder", and
     # "Replace" arguments, respectively.
 
     # Make the pixel at (x,y) transparent.
@@ -704,6 +778,20 @@ class Image
         self
     end
 
+    # Convenience method to resize retaining the aspect ratio.
+    # (Thanks to Robert Manni!)
+    def resize_to_fit(cols, rows)
+        change_geometry(Geometry.new(cols, rows)) do |ncols, nrows|
+            resize(ncols, nrows)
+        end
+    end
+
+    def resize_to_fit!(cols, rows)
+        change_geometry(Geometry.new(cols, rows)) do |ncols, nrows|
+            resize!(ncols, nrows)
+        end
+    end
+
     # Replace matching neighboring pixels with texture pixels
     def texture_floodfill(x, y, texture)
         target = pixel_color(x, y)
@@ -715,47 +803,6 @@ class Image
         texture_flood_fill(border_color, texture, x, y, FillToBorderMethod)
     end
 
-    # Retrieve EXIF data by entry or all. If one or more entry names specified,
-    # return the values associated with the entries. If no entries specified,
-    # return all entries and values. The return value is an array of [name,value]
-    # arrays.
-    def get_exif_by_entry(*entry)
-        ary = Array.new
-        if entry.length == 0
-            exif_data = self['EXIF:*']
-            if exif_data
-                exif_data.split("\n").each { |exif| ary.push(exif.split('=')) }
-            end
-        else
-            entry.each do |name|
-                rval = self["EXIF:#{name}"]
-                ary.push([name, rval])
-            end
-        end
-        return ary
-    end
-
-    # Retrieve EXIF data by tag number or all tag/value pairs. The return value is a hash.
-    def get_exif_by_number(*tag)
-        hash = Hash.new
-        if tag.length == 0
-            exif_data = self['EXIF:!']
-            if exif_data
-                exif_data.split("\n").each do |exif|
-                    tag, value = exif.split('=')
-                    tag = tag[1,4].hex
-                    hash[tag] = value
-                end
-            end
-        else
-            tag.each do |num|
-                rval = self["EXIF:#{'#%04X' % num}"]
-                hash[num] = rval == 'unknown' ? nil : rval
-            end
-        end
-        return hash
-    end
-
     # Construct a view. If a block is present, yield and pass the view
     # object, otherwise return the view object.
     def view(x, y, width, height)
